import 'package:cpt_job/modules/job_list_detail/job_list_detail_page.dart';
import 'package:domain/entity/response/job_list_entity.dart';
import 'package:domain/entity/response/job_list_index_entity.dart';
import 'package:domain/repository/job_repository.dart';
import 'package:get/get.dart';
import 'package:plugin_platform/engine/dialog/dialog_engine.dart';
import 'package:plugin_platform/http/dio/dio_cancelable_mixin.dart';
import 'package:plugin_platform/http/http_result.dart';
import 'package:shared/utils/date_time_utils.dart';
import 'package:widgets/load_state_layout.dart';
import 'package:widgets/widget_export.dart';

import '../job_applied/job_applied_page.dart';
import 'job_list_filter.dart';
import 'job_list_state.dart';

class JobListController extends GetxController with DioCancelableMixin {
  final JobRepository _jobRepository = Get.find();
  final JobListState state = JobListState();

  var _curPage = 1;
  var _needShowPlaceholder = true;

  //页面PlaceHolder的展示
  LoadState loadingState = LoadState.State_Success;
  String? errorMessage;

  //刷新页面状态
  void changeLoadingState(LoadState state) {
    loadingState = state;
    update();
  }

  // Refresh 控制器
  final EasyRefreshController refreshController = EasyRefreshController(
    controlFinishRefresh: true,
    controlFinishLoad: true,
  );

  // Refresh 刷新事件
  Future onRefresh() async {
    _curPage = 1;
    fetchNotifyList();
  }

  // Refresh 加载事件
  Future loadMore() async {
    _curPage++;
    fetchNotifyList();
  }

  // 重试请求
  Future retryRequest() async {
    _curPage = 1;
    _needShowPlaceholder = true;
    fetchNotifyList();
  }

  /// 获取服务器数据，通知消息列表
  Future fetchNotifyList() async {
    if (_needShowPlaceholder) {
      changeLoadingState(LoadState.State_Loading);
    }

    // 并发执行两个请求
    var futures = [
      _jobRepository.fetchJobListTable(
        state.keyword,
        DateTimeUtils.formatDate(state.selectedStartDate, format: "yyyy-MM-dd"),
        DateTimeUtils.formatDate(state.selectedEndDate, format: "yyyy-MM-dd"),
        state.selectedStatusId,
        state.selectedDepartmentId,
        curPage: _curPage,
        cancelToken: cancelToken,
      ),
      state.indexOptions == null
          ? _jobRepository.fetchJobListIndex(
              cancelToken: cancelToken,
            )
          : Future(() => HttpResult(isSuccess: true).convert(data: state.indexOptions!)),
    ];

    //拿到结果
    var results = await Future.wait(futures);
    var listResult = results[0] as HttpResult<JobListEntity>;
    var optionResult = results[1] as HttpResult<JobListIndexEntity>;

    //选项数据
    if (state.indexOptions == null && optionResult.isSuccess) {
      state.indexOptions = optionResult.data!;
    }

    // 处理数据
    if (listResult.isSuccess) {
      handleList(listResult.data?.rows);
    } else {
      errorMessage = listResult.errorMsg;
      changeLoadingState(LoadState.State_Error);
    }

    // 最后赋值
    _needShowPlaceholder = false;
  }

  // 处理数据与展示的逻辑
  void handleList(List<JobListRows>? list) {
    if (list != null && list.isNotEmpty) {
      //有数据，判断是刷新还是加载更多的数据
      if (_curPage == 1) {
        //刷新的方式
        state.datas.clear();
        state.datas.addAll(list);
        refreshController.finishRefresh();

        //更新展示的状态
        changeLoadingState(LoadState.State_Success);
      } else {
        //加载更多
        state.datas.addAll(list);
        refreshController.finishLoad();
        update();
      }
    } else {
      if (_curPage == 1) {
        //展示无数据的布局
        state.datas.clear();
        changeLoadingState(LoadState.State_Empty);
        refreshController.finishRefresh();
      } else {
        //展示加载完成，没有更多数据了
        refreshController.finishLoad(IndicatorResult.noMore);
      }
    }
  }

  // 执行搜索
  void doSearch(String keyword) {
    state.keyword = keyword;
    //赋值之后刷新
    refreshController.callRefresh();
  }

  // 清空搜索条件
  void resetFiltering() {
    state.keyword = "";
    state.searchController.text = "";

    state.selectedStartDate = null;
    state.selectedEndDate = null;
    state.selectedStatusId = null;
    state.selectedDepartmentId = null;

    //赋值之后刷新
    refreshController.callRefresh();
  }

  //展示筛选的弹窗
  void showFilterDialog() {
    if (state.indexOptions != null) {
      DialogEngine.show(
        widget: JobListFilter(
          optionResult: state.indexOptions!,
          selectedStartDate: state.selectedStartDate,
          selectedEndDate: state.selectedEndDate,
          selectedStatusId: state.selectedStatusId,
          selectedDepartmentId: state.selectedDepartmentId,
          onFilterAction: (startDate, endDate, statusId, departmentId) {
            state.selectedStartDate = startDate;
            state.selectedEndDate = endDate;
            state.selectedStatusId = statusId;
            state.selectedDepartmentId = departmentId;

            //赋值之后刷新
            refreshController.callRefresh();
          },
        ),
        position: DialogPosition.top,
        animType: DialogAnimation.fade,
      );
    }
  }

  @override
  void onReady() {
    super.onReady();
    fetchNotifyList();
  }

  @override
  void onClose() {
    state.datas.clear();
    super.onClose();
  }

  /// 去详情页面
  void gotoJobDetailPage(JobListRows data) {
    JobListDetailPage.startInstance(data.jobId.toString());
  }

  /// 去已申请的成员列表
  void gotoJobAppliedPage(JobListRows data) {
    JobAppliedPage.startInstance(data.jobId.toString());
  }
}
