import 'package:cpt_job_sg/modules/revise_add_edit/revise_add_edit_page.dart';
import 'package:domain/entity/response/revise_index_s_g_entity.dart';
import 'package:domain/entity/response/revise_list_s_g_entity.dart';
import 'package:domain/repository/job_sg_repository.dart';
import 'package:get/get.dart';
import 'package:plugin_platform/engine/dialog/dialog_engine.dart';
import 'package:plugin_platform/engine/notify/notify_engine.dart';
import 'package:plugin_platform/engine/toast/toast_engine.dart';
import 'package:plugin_platform/http/dio/dio_cancelable_mixin.dart';
import 'package:plugin_platform/http/http_result.dart';
import 'package:shared/utils/date_time_utils.dart';
import 'package:widgets/dialog/app_default_dialog.dart';
import 'package:widgets/load_state_layout.dart';
import 'package:widgets/widget_export.dart';

import '../revise_log/revise_log_page.dart';
import 'revise_list_filter.dart';
import 'revise_list_state.dart';

class ReviseListController extends GetxController with DioCancelableMixin {
  final JobSGRepository _jobRepository = Get.find();
  final ReviseListState state = ReviseListState();

  var _curPage = 1;
  var _needShowPlaceholder = true;

  //页面PlaceHolder的展示
  LoadState loadingState = LoadState.State_Success;
  String? errorMessage;

  //刷新页面状态
  void changeLoadingState(LoadState state) {
    loadingState = state;
    update();
  }

  // Refresh 控制器
  final EasyRefreshController refreshController = EasyRefreshController(
    controlFinishRefresh: true,
    controlFinishLoad: true,
  );

  // Refresh 刷新事件
  Future onRefresh() async {
    _curPage = 1;
    fetchNotifyList();
  }

  // Refresh 加载事件
  Future loadMore() async {
    _curPage++;
    fetchNotifyList();
  }

  // 重试请求
  Future retryRequest() async {
    _curPage = 1;
    _needShowPlaceholder = true;
    fetchNotifyList();
  }

  /// 获取服务器数据，通知消息列表
  Future fetchNotifyList() async {
    if (_needShowPlaceholder) {
      changeLoadingState(LoadState.State_Loading);
    }

    // 并发执行两个请求
    var futures = [
      _jobRepository.fetchReviseList(
        null,
        state.selectedStatusId,
        state.keyword,
        DateTimeUtils.formatDate(state.selectedStartDate, format: "yyyy-MM-dd"),
        DateTimeUtils.formatDate(state.selectedEndDate, format: "yyyy-MM-dd"),
        curPage: _curPage,
        cancelToken: cancelToken,
      ),
      state.indexOptions == null
          ? _jobRepository.fetchReviseIndex(
              cancelToken: cancelToken,
            )
          : Future(() => HttpResult(isSuccess: true).convert(data: state.indexOptions!)),
    ];

    //拿到结果
    var results = await Future.wait(futures);
    var listResult = results[0] as HttpResult<ReviseListSGEntity>;
    var optionResult = results[1] as HttpResult<ReviseIndexSGEntity>;

    //选项数据
    if (state.indexOptions == null && optionResult.isSuccess) {
      state.indexOptions = optionResult.data!;
    }

    // 处理数据
    if (listResult.isSuccess) {
      handleList(listResult.data?.rows);
    } else {
      errorMessage = listResult.errorMsg;
      changeLoadingState(LoadState.State_Error);
    }

    // 最后赋值
    _needShowPlaceholder = false;
  }

  // 处理数据与展示的逻辑
  void handleList(List<ReviseListSGRows>? list) {
    if (list != null && list.isNotEmpty) {
      //有数据，判断是刷新还是加载更多的数据
      if (_curPage == 1) {
        //刷新的方式
        state.datas.clear();
        state.datas.addAll(list);
        refreshController.finishRefresh();

        //更新展示的状态
        changeLoadingState(LoadState.State_Success);
      } else {
        //加载更多
        state.datas.addAll(list);
        refreshController.finishLoad();
        update();
      }
    } else {
      if (_curPage == 1) {
        //展示无数据的布局
        state.datas.clear();
        changeLoadingState(LoadState.State_Empty);
        refreshController.finishRefresh();
      } else {
        //展示加载完成，没有更多数据了
        refreshController.finishLoad(IndicatorResult.noMore);
      }
    }
  }

  // 执行搜索
  void doSearch(String keyword) {
    state.keyword = keyword;
    //赋值之后刷新
    refreshController.callRefresh();
  }

  // 清空搜索条件
  void resetFiltering() {
    state.keyword = "";
    state.searchController.text = "";

    state.selectedStartDate = null;
    state.selectedEndDate = null;
    state.selectedStatusId = null;

    //赋值之后刷新
    refreshController.callRefresh();
  }

  //展示筛选的弹窗
  void showFilterDialog() {
    if (state.indexOptions != null) {
      DialogEngine.show(
        widget: ReviseListFilter(
          optionResult: state.indexOptions!,
          selectedStartDate: state.selectedStartDate,
          selectedEndDate: state.selectedEndDate,
          selectedStatusId: state.selectedStatusId,
          onFilterAction: (startDate, endDate, statusId) {
            state.selectedStartDate = startDate;
            state.selectedEndDate = endDate;
            state.selectedStatusId = statusId;

            //赋值之后刷新
            refreshController.callRefresh();
          },
        ),
        position: DialogPosition.top,
        animType: DialogAnimation.fade,
      );
    }
  }

  @override
  void onReady() {
    super.onReady();
    fetchNotifyList();
  }

  @override
  void onClose() {
    state.datas.clear();
    super.onClose();
  }

  /// 去日志页面
  void gotoReviseLogsPage(int index) {
    ReviseLogPage.startInstance(state.datas[index].reviseId);
  }

  /// 去详情页面
  void gotoReviseEditPage(int index) {
    ReviseAddEditPage.startInstance(1, state.datas[index].reviseId,(result){
      if (result is String){
        //编辑之后刷新当前Item
        fetchItemByIdAndRefreshItem(result);
      }
    });
  }

  /// 去编辑页面
  void gotoReviseDetailPage(int index) {
    ReviseAddEditPage.startInstance(2, state.datas[index].reviseId,null);
  }

  //执行取消操作
  void doRecallAction(int index) {
    DialogEngine.show(
        widget: AppDefaultDialog(
      title: "Confirmation".tr,
      message: "Are you sure you want to recall this revise?".tr,
      confirmAction: () {
        _requestRecall(index);
      },
    ));
  }

  //执行删除操作
  void doDeleteAction(int index) {
    DialogEngine.show(
        widget: AppDefaultDialog(
      title: "Confirmation".tr,
      message: "Are you sure you want to delete this revise?".tr,
      confirmAction: () {
        _requestDeactivate(index);
      },
    ));
  }

  // 请求接口删除Job
  void _requestDeactivate(int index) async {
    var result = await _jobRepository.deleteRevise(state.datas[index].reviseId, cancelToken: cancelToken);

    if (result.isSuccess) {
      NotifyEngine.showSuccess("Successful".tr);

      if (state.datas.length <= 1) {
        refreshController.callRefresh();
      } else {
        state.datas.removeAt(index);
        update();
      }
    } else {
      ToastEngine.show(result.errorMsg ?? "Network Load Error".tr);
    }
  }

  void _requestRecall(int index) async {
    var result = await _jobRepository.recallRevise(state.datas[index].reviseId, cancelToken: cancelToken);

    if (result.isSuccess) {
      NotifyEngine.showSuccess("Successful".tr);

      fetchItemByIdAndRefreshItem(state.datas[index].reviseId ?? "");
    } else {
      ToastEngine.show(result.errorMsg ?? "Network Load Error".tr);
    }
  }

  /// 根据ID获取Item对象，用于刷新
  void fetchItemByIdAndRefreshItem(String ids) async {
    var result = await _jobRepository.fetchReviseListByIds(
      ids,
      cancelToken: cancelToken,
    );

    //处理数据
    if (result.isSuccess) {
      var data = result.data;
      if (data != null && data.rows.isNotEmpty) {
        List<ReviseListSGRows> newItem = data.rows;

        // 创建一个 Map 来加速查找
        Map<String, ReviseListSGRows> newItemMap = {for (var item in newItem) item.reviseId ?? "": item};

        // 遍历 state.datas 进行替换
        for (int i = 0; i < state.datas.length; i++) {
          if (newItemMap.containsKey(state.datas[i].reviseId)) {
            state.datas[i] = newItemMap[state.datas[i].reviseId]!;
          }
        }

        //刷新
        update();
      }
    } else {
      ToastEngine.show(result.errorMsg ?? "Network Load Error".tr);
    }
  }
}
